from typing import Literal as _Literal, Union as _Union


UnknownErrcode = _Literal["unknown"]

LiqpayAntiFraudErrcode = _Literal["limit", "frod", "decline"]

LiqpayRequestErrcode = _Literal[
    "err_action",
    "invalid_signature",
    "public_key_not_found",
    "order_id_empty",
    "amount_limit",
    "wrong_amount_currency",
]

LiqpayExpireErrcode = _Literal[
    "expired_phone",
    "expired_3ds",
    "expired_otp",
    "expired_cvv",
    "expired_p24",
    "expired_sender",
    "expired_pin",
    "expired_ivr",
    "expired_captcha",
    "expired_password",
    "expired_senderapp",
    "expired_prepared",
    "expired_mp",
    "expired_qr",
]

LiqpayPaymentErrcode = _Literal[
    "payment_err_sender_card",
    "payment_processing",
    "payment_err_status",
    "payment_not_found",
    "payment_not_subscribed",
    "payment_err_type",
]

LiqpayNonFinancialErrcode = _Union[
    _Literal[
        "err_auth",
        "err_cache",
        "err_sms_send",
        "err_sms_otp",
        "err_shop_not_agent",
        "err_card_def_notfound",
        "err_no_card_token",
        "err_card_liqpay_def",
        "err_card_type",
        "err_card_country",
        "err_limit_amount",
        "err_payment_amount_limit",
        "err_payment_discount",
        "err_wallet",
        "err_get_verify_code",
        "err_verify_code",
        "err_path",
        "err_payment_cash_acq",
        "err_split_amount",
        "err_card_receiver_def",
        "err_amount_hold",
        "err_access",
        "err_blocked",
        "err_empty",
        "err_empty_phone",
        "err_missing",
        "err_wrong",
        "err_wrong_currency",
        "err_phone",
        "err_card",
        "err_card_bin",
        "err_terminal_notfound",
        "err_commission_notfound",
        "err_payment_create",
        "err_mpi",
        "err_currency_is_not_allowed",
        "err_look",
        "err_mods_empty",
        "err_payment_currency",
        "err_payment_exchangerates",
        "err_signature",
        "err_api_action",
        "err_api_callback",
        "err_api_ip",
        "user_not_found",
        "wait_info",
        "order_id_duplicate",
        "shop_blocked",
        "shop_not_active",
        "5",
    ],
    LiqpayExpireErrcode,
    LiqpayRequestErrcode,
    LiqpayPaymentErrcode,
]

LiqpayFinancialErrcode = _Literal[
    "90",
    "101",
    "102",
    "103",
    "104",
    "105",
    "106",
    "107",
    "108",
    "109",
    "110",
    "111",
    "112",
    "113",
    "114",
    "115",
    "2903",
    "2915",
    "3914",
    "9851",
    "9852",
    "9854",
    "9855",
    "9857",
    "9859",
    "9860",
    "9861",
    "9863",
    "9867",
    "9868",
    "9872",
    "9882",
    "9886",
    "9961",
    "9989",
]

LiqPayErrcode = _Union[
    UnknownErrcode,
    LiqpayAntiFraudErrcode,
    LiqpayFinancialErrcode,
    LiqpayNonFinancialErrcode,
]
